@@ -13,12 +13,15 @@ module System.Console.Argument
   , integer
   
   -- * Option descriptions
+  , Option
   , option
   ) where
 
 
-import           Data.Char      (toLower)
-import           Data.List.HT   (viewR)
+import           System.Console.Internal (Option(Option),Identifier(Short,Long))
+
+import           Data.Char        (toLower)
+import           Data.List.HT     (viewR)
 import qualified Data.Map              as Map
 import qualified System.Console.GetOpt as GetOpt
 import qualified Text.Parsec.Extra     as P
@@ -48,20 +51,34 @@ data Type a
 instance Functor Type where
   fmap f t = t { parser = fmap f . parser t, defaultValue = fmap f (defaultValue t) }
 
--- | Create an option description. You need this to describe the options your
--- command uses; see 'System.Console.Command.Command'.
+-- | Create an option description.
+-- 
+-- Options can have arguments, as in @myprogram --foo=bar@, where @bar@
+-- is the argument to @foo@. These arguments have types, dictated by the
+-- particular option; this type is the third parameter to @option@.
 option
-  :: (a -> s) -- ^ Function that creates a setting (of type @s@) from the option argument.
-  -> [Char]   -- ^ List of short option characters.
-  -> [String] -- ^ List of long option strings.
-  -> Type a   -- ^ Type of option argument.
-  -> String   -- ^ Description.
-  -> GetOpt.OptDescr (Either String s) -- ^ The resulting option description.
-option inj short long t description = case defaultValue t of
-  Nothing -> GetOpt.Option short long (GetOpt.ReqArg (                        fmap inj . parser t) (name t)) description
-  Just a  -> GetOpt.Option short long (GetOpt.OptArg (maybe (Right $ inj a) $ fmap inj . parser t) (name t)) description
+  :: [Char]     -- ^ List of short option characters.
+  -> [String]   -- ^ List of long option strings.
+  -> Type a     -- ^ Type of option argument.
+  -> a          -- ^ Default value.
+  -> String     -- ^ Description.
+  -> Option a   -- ^ The resulting option description.
+option short long t def description = let
+  identifier = if null short then Long (head long) else Short (head short)
+ in Option
+  identifier
+  (GetOpt.Option
+    short
+    long
+    (maybe
+      (GetOpt.ReqArg ((,) identifier . Just) $ name t)
+      (const $ GetOpt.OptArg ((,) identifier) $ name t)
+      (defaultValue t))
+    description)
+  def
+  (maybe (maybe (Left "Option argument missing.") Right $ defaultValue t) (parser t))
 
--- Argument types.
+-- Common argument types.
 
 optional
   :: a      -- ^ Default value.
@@ -73,7 +90,7 @@ optional x t = t { defaultValue = Just x }
 string :: Type String
 string = Type Right "STRING" Nothing
 
--- | A boolean. Argument can be \"1\",\"0\",\"true\",\"false\",\"on\",\"off\".
+-- | A boolean. Argument can be \"1\",\"0\",\"true\",\"false\",\"on\",\"off\",\"yes\",\"no\".
 boolean :: Type Bool
 boolean = Type
   {
@@ -82,7 +99,7 @@ boolean = Type
   , defaultValue = Just True
   }
  where
-  m = Map.fromList [("1",True),("0",False),("true",True),("false",False),("on",True),("off",False)]
+  m = Map.fromList [("1",True),("0",False),("true",True),("false",False),("on",True),("off",False),("yes",True),("no",False)]
   e y = Left $ "Argument " ++ show y ++ " could not be recognised as a boolean."
 
 -- | A natural number (in decimal).
